from bs4 import BeautifulSoup

class Extract:
    def __init__(self, html):
        self.html = html
        self.soup = self._get_soup()

    def _get_soup(self):
        soup = BeautifulSoup(self.html, 'html.parser')
        return soup

    def get_main(self):
        rows = self.soup.find("div", {"class": "main"})
        return rows

    @staticmethod
    def get_rows(soup):
        rows = soup.find_all("div", {"class": "row"})
        return rows

    @staticmethod
    def get_columns(soup):
        columns = soup.find_all("div", {"class": "col-sm-6"})
        return columns

    @staticmethod
    def get_p(soup):
        p = soup.find("p")
        return p


class Convert:
    def __init__(self, html):
        self.ext = Extract(html)

    def retrieve_first_items(self, col):
        p = self.ext.get_p(col)
        ol = self.find_next(p, 'ol')
        lis = self.get_items(ol)
        return p, ol, lis

    def retrieve_next_items(self, ol):
        col_next_p = self.find_next(ol, 'p')
        next_ol = self.find_next(col_next_p, 'ol')
        lis = self.get_items(next_ol)
        return col_next_p, next_ol, lis

    @staticmethod
    def get_items(soup):
        lis = soup.find_all("li")
        return lis

    @staticmethod
    def find_next(soup, type):
        next = soup.findNext(type)
        return next



from bs4.element import Tag
from .convert import Extract, Convert

class Columns:
    def __init__(self, html):
        self.ext = Extract(html)
        self.columns = self.get_columns()

    def get_columns(self):
        main = self.ext.get_main()
        columns = self.ext.get_columns(main)
        return columns


class SectionBase:
    def __init__(self, html:str, columns:Tag):
        self.conv = Convert(html)

    def __post_init__(self, title_count:int):
        for i in range(title_count-1):
            try:
                self._get_items(i)
            except AttributeError:
                break

    @property
    def items_1(self) -> tuple[str, Tag]:
        pass

    def _get_items(self, columns:Tag):
        pass


class Section(SectionBase):
    def __init__(self, html, columns, title_count):
        super().__init__(html, columns)
        self.p1, self.ol1, self.lis1 = self.conv.retrieve_first_items(columns)
        super().__post_init__(title_count)

    @property
    def items_1(self):
        return self.p1.find('strong').text, self.lis1

    def _get_items(self, count):
        this_ol = getattr(self, 'ol'+str(count+1))
        new_p, new_ol, new_lis = self.conv.retrieve_next_items(this_ol)
        self._set_attrs(new_p, new_ol, new_lis, count)

    def _set_attrs(self, new_p, new_ol, new_lis, count):
        setattr(self, 'p'+str(count+2), new_p)
        setattr(self, 'ol'+str(count+2), new_ol)
        setattr(self, 'lis'+str(count+2), new_lis)
        setattr(self, 'items_'+str(count+2), (new_p.find('strong').text, new_lis))



from .to_csv.convert import Convert
from .to_csv.save import LimitVTOL, CSVHandler
from .utils.config import DETAIL_PATH

class SaveToCsv:
    def __init__(self):
        self.csv = CSVHandler()
        self.limit = LimitVTOL()
        self.converter = self.get_converter()

    @property
    def data(self):
        data = self.converter.converted_data
        return data

    @property
    def limited_by_params(self):
        limited_data = self.limit.limit_by_params(
            self.converter.converted_data
        )
        return limited_data

    @property
    def limited_by_capacity(self):
        limited_data = self.limit.limit_capacity(
            self.converter.converted_data,
            min_cap=3,
            max_cap=6
        )
        return limited_data

    @property
    def limited_by_params_by_capacity(self):
        limited_data = self.limit.limit_capacity(
            self.limited_by_params,
            min_cap=3,
            max_cap=6
        )
        return limited_data

    def save(self, name, data):
        self.csv.write(name, data)

    @staticmethod
    def get_converter():
        converter = Convert(DETAIL_PATH)
        converter.execute()
        return converter



import requests as req

class Request:
    def __init__(self, url):
        self.url = url
        self.headers = self.get_headers()

    def send_request(self):
        print("sending request to (", self.url, ") ...")
        res = req.get(self.url, self.headers)
        return res

    @staticmethod
    def get_headers():
        headers = dict()
        headers['User-Agent'] = 'Mozilla/5.0 (X11; Linux x86_64; rv:102.0) Gecko/20100101 Firefox/102.0'
        return headers



import re, csv
from collections import OrderedDict
from evtol.utils import parameters
from .convert import Convert

class LimitVTOL:
    def __init__(self):
        self.analyzer = TextAnalyzer()
        self.params = [var for var in dir(parameters) if not var.startswith('__')]

    def limit_by_params(self, data):
        converted_data = {}
        for title, vtol_pair in data.items():
            converted_data[title] = {}
            for vtol, detail in vtol_pair.items():
                converted_data[title][vtol] = self.populate_params()
                for detail_key, detail_value in detail.items():
                    for param in self.params:
                        attr = getattr(parameters, param)
                        if detail_key in attr:
                            converted_data[title][vtol][param] = detail_value
        return converted_data

    def limit_capacity(self, converted_data, min_cap, max_cap):
        limited_result = dict()
        for title, vtol_pair in converted_data.items():
            limited_result[title] = dict()
            for vtol, detail in vtol_pair.items():
                for detail_key, detail_value in detail.items():
                    if detail_key == 'capacity':
                        if self.analyzer.has_numbers(detail_value):
                            detail_numbers = self.analyzer.get_numbers(detail_value)
                            if len(detail_numbers)==1:
                                if detail_numbers[0]<min_cap:
                                    continue
                            if max(detail_numbers)<=min_cap:
                                continue
                            if detail_value.find('No passengers')>=0:
                                continue
                            if max(detail_numbers)<=max_cap:
                                limited_result[title][vtol] = converted_data[title][vtol]
                                limited_result[title][vtol]['capacity'] = detail_value
        return limited_result

    def populate_params(self):
        result = OrderedDict()
        for param in self.params:
            result[param] = None
        return result

    @staticmethod
    def get_vtol_count(full_data):
        count = 0
        for vtol_pair in full_data.values():
            count += len(vtol_pair)
        return count


class TextAnalyzer:
    @staticmethod
    def has_numbers(inputString):
        if inputString is not None:
            return any(char.isdigit() for char in inputString)

    @staticmethod
    def get_numbers(text):
        result = []
        numbers = re.findall(r'\b\d{1,3}(?:,\d{3})*(?:\.\d+)?(?!\d)', text)
        for number in numbers:
            int_number = int(number.replace(',', ''))
            result.append(int_number)
        return result


class CSVHandler:
    def __init__(self):
        self._writer = None
        self.fieldnames = list()
        self.params = [var for var in dir(parameters) if not var.startswith('__')]

    def write(self, file_name:str, converted_data:dict):
        with open(file_name + '.csv', mode='w') as csv_file:
            self._create_headers(csv_file)
            for title, vtol_pair in converted_data.items():
                for vtol, detail in vtol_pair.items():
                    self._create_row(vtol, detail)

    def _create_headers(self, csv_file):
        self._add_vtol_name()
        self.writer = csv.DictWriter(csv_file, fieldnames=self.fieldnames)
        self.writer.writeheader()

    def _create_row(self, vtol, data):
        data['vtol'] = vtol
        self.writer.writerow(data)

    def _add_vtol_name(self):
        self.fieldnames = ['vtol']
        self.fieldnames.extend(self.params)



def progressBar(iterable, prefix = '', suffix = '', decimals = 1, length = 100, fill = '█', printEnd = "\r"):
    total = len(iterable)
    def printProgressBar (iteration):
        percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
        filledLength = int(length * iteration // total)
        bar = fill * filledLength + '-' * (length - filledLength)
        print(f'\r{prefix} |{bar}| {percent}% {suffix}', end = printEnd)
    printProgressBar(0)
    for i, item in enumerate(iterable):
        yield item
        printProgressBar(i + 1)
    print()



import os
from time import sleep
from .scrape import Request
from evtol.utils.config import DETAIL_PATH

class SaveVTOL:
    def __init__(self, title, lis):
        self.title = self.suitable_file_name(title)
        self.lis = lis

    def save(self):
        item_number = 1
        for li in self.lis:
            path = self.check_directory(self.title)
            name = self.suitable_file_name(li.find('a').text)
            file_name = path + str(item_number) + "." + name  + ".html"
            if os.path.isfile(file_name):
                if os.path.getsize(file_name) > 0:
                    item_number += 1
                    continue
            with open(file_name, 'w') as f:
                url = li.find('a').get('href')
                req = Request(url)
                response = req.send_request()
                f.write(response.text)
            item_number += 1
            sleep(1)

    @staticmethod
    def suitable_file_name(name):
        name = name.replace(' ', '')
        name = name.replace('/', '_')
        return name

    @staticmethod
    def check_directory(title):
        path = DETAIL_PATH + title + "/"
        is_exist = os.path.exists(path)
        if not is_exist:
            os.makedirs(path)
        return path



from setuptools import setup, find_packages

setup(
    name='evtol_crawler',
    version='0.0.1',
    license='MIT',
    author="Benyamin Kosari",
    author_email='benyaminusc@gmail.com',
    packages=find_packages('source'),
    package_dir={'': 'source'},
    url='https://github.com/benyaminkosari/evtol_crawler',
    keywords='evtol crawler',
    install_requires=[
          'requests',
          'bs4',
      ],
)



import os, sys
from bs4 import BeautifulSoup
from evtol.utils.progress import progressBar
from collections import OrderedDict

class Extract:
    def __init__(self, html, path):
        self.html = html
        self.path = path
        self.soup = self._get_soup()

    def _get_soup(self):
        soup = BeautifulSoup(self.html, 'html.parser')
        return soup

    def get_main(self):
        content = self.soup.find("div", {"class": "content-page"})
        try:
            main = content.find("div", {"class": "row"})
        except:
            try:
                h1 = self.soup.find("h1").text
            except:
                return None
            if h1 is not None:
                if h1.find("Something went wrong")>1:
                    main = None
                else:
                    print("ERROR:", self.path)
                    sys.exit(1)
            else:
                print("ERROR:", self.path)
                sys.exit(1)
        return main

    @staticmethod
    def get_specifications(main):
        if main is not None:
            output_ps = []
            ps = main.find_all("p")
            for p in ps:
                strong = p.find('strong')
                if strong is not None:
                    strong = strong.text.replace(' ', '')
                    if strong.startswith("Specification"):
                        ul = p.next_sibling.next_sibling
                        if ul is None:
                            print("ERROR: EMPTY UL")
                        output_ps.append(ul)
            return output_ps

class Convert:
    converted_data = OrderedDict()

    def __init__(self, path):
        self.path = path

    def execute(self):
        titles = os.listdir(self.path)
        for title in titles:
            files_path = self.path + "/" + title
            file_names = os.listdir(files_path)
            file_names = self.sort_files(file_names)
            self.converted_data[title] = {}
            for file_name in progressBar(file_names, prefix=title , suffix='', fill='=', length=50):
                full_path = files_path + "/" + file_name
                with open(full_path, "r") as file:
                    html = file.read()
                    self.conv_li(title, file_name, html)

    def conv_li(self, title, file_name, html):
        ext = Extract(html, self.path)
        main = ext.get_main()
        res = ext.get_specifications(main)
        vtol = file_name.replace('.html', '')
        if res:
            res = res[0]
            lis = res.find_all('li')
            li_dict = {}
            self.converted_data[title][vtol] = OrderedDict()
            for li in lis:
                try:
                    self.conv_pair(li_dict, li, ':')
                except IndexError:
                    try:
                        self.conv_pair(li_dict, li, ';')
                    except:
                        continue
                self.converted_data[title][vtol] = li_dict

    def conv_pair(self, li_dict, li, item):
        li_pair = li.text.split(':')
        li_topic = li_pair[0]
        li_dict[li_topic] = self.clean(li_pair[1])

    @staticmethod
    def sort_files(files):
        temp_dict = dict()
        sorted_files = list()
        for i, file in enumerate(files):
            temp_dict[i] = int(file.split('.')[0])
        sorted_dict = OrderedDict(sorted(temp_dict.items(), key=lambda item: item[1]))
        for key, value in sorted_dict.items():
            sorted_files.append(files[key])
        return sorted_files

    @staticmethod
    def clean(detail):
        return detail.replace(';', ',')



import os
from .downloader.scrape import Request
from .downloader.save import SaveVTOL
from .downloader.assort import Columns, Section
from .utils.config import HOMEPAGE_PATH, HOMEPAGE_URL

class Download:
    def __init__(self):
        self.html = self.get_home() or self.save_home()
        self.col = Columns(self.html)
        self.sec1 = Section(self.html, self.col.columns[0], 2)
        self.sec2 = Section(self.html, self.col.columns[1], 3)

    def all(self):
        self.save_section(self.sec1)
        self.save_section(self.sec2)

    def save_section(self, sec):
        items = self._get_items(sec)
        self.save_items(sec, items)

    def get_home(self):
        try:
            with open(HOMEPAGE_PATH, "r") as file:
                html = file.read()
                return html
        except FileNotFoundError:
            return None

    def save_home(self):
        req = Request(HOMEPAGE_URL)
        res = req.send_request()
        self.check_html_dirs()
        with open(HOMEPAGE_PATH, "w") as file:
            file.write(res.text)
        return res.text

    @staticmethod
    def save_items(sec, items):
        for item in items:
            topic, detail = getattr(sec, item)
            save_handler = SaveVTOL(topic, detail)
            save_handler.save()

    @staticmethod
    def _get_items(sec):
        return [var for var in dir(sec) if var.startswith('items')]

    @staticmethod
    def check_html_dirs():
        if not os.path.exists("html_samples"):
            os.makedirs("html_samples/main")
            os.makedirs("html_samples/detail")



height = [
    'Overall height',
    'Fuselage height',
    'Height',
    'Height (to top of tail)',
    'Height of Aircraft',
    'Vehicle height',
    'Aircraft height'
]

length = [
    'Fuselage length',
    'Overall length',
    'Length',
    'Length of fuselage',
    'Total length',
    'Vehicle length',
    'Aircraft length'
]

width = [
    'Fuselage width',
    'Vehicle width',
    'Width',
    'Aircraft breadth'
]

size = [
    'Vehicle size',
    'Size',
    'Airframe size'
]

length_and_width = [
    'Length and width',
    'Width and Length',
    'Width and length'
]

dimensions = [
    'Dimensions',
]

wingspan = [
    'Wing span',
    'Wings span',
    'Wingspan',
    'Wingspan (full)'
]

passenger_cabin_length = [
    'Passenger cabin length',
]

passenger_cabin_width = [
    'Passenger cabin width',
]

capacity = [
    'Passenger capacity',
    'Capacity'
]

battery_type = [
    'Battery type',
]

battery_capacity = [
    'Battery capacity',
    'Battery Specific Energy'
]

battery_weight = [
    'Battery weight',
]

fuselage_material = [
    'Fuselage material',
    'Fuselage\xa0material'
]

horse_power = [
    'Horse power',
    'Maximum power output ',
    'Power',
    'Propulsion power',
    'Total power'
]

thrust = [
    'Total system thrust',
    'Maximum thrust',
    'Thrust',
    'Top thrust',
    'Average thrust'
]

static_thrust = [
    'Static Thrust',
]

power_to_weight_ratio = [
    'Power to weight ratio',
]

thrust_ratio = [
    'Thrust ratio',
]

wing_loading = [
    'Wing loading',
]

ceiling = [
    'Ceiling',
]

hover_time = [
    'Hover flight time',
    'Hover time',
    'Hovering-only flight time'
]

landing_gear = [
    'Landing gear',
    'Landing gear configuration'
]

mtow = [
    'MTO weight',
    'MTOW',
    'Maximum Takeoff weight',
    'Max Takeoff Weight',
    'Max takeoff weight',
    'Maximum Take-Off Weight (MTOW)',
    'Maximum take off weight',
    'Maximum take-off weight',
    'Maximum takeoff weight',
    'Maximum takeoff weight (MTOW)',
    'Maximum takeoff weight (fully electric)',
    'Maximum takeoff weight (hybrid-electric)',
    'Maximum vertical takeoff weight',
    'Maximum weight',
    'Takeoff weight'
]

mtom = [
    'Maximum take-off mass (MTOM)',
    'Maximum takeoff mass'
]

gross_weight = [
    'Maximum gross weight',
    'Gross weight'
]

empty_weight = [
    'Operating weight empty (OWE)',
    'Total empty weight',
    'Weight of aircraft empty',
    'Empty Weight',
    'Empty weight',
    'Empty weight without batteries',
    'Aircraft empty weight'
]

baggage = [
    'Baggage',
]

weight = [
    'Weight',
]

payload = [
    'Max Payload',
    'Maximum Payload',
    'Maximum cargo load',
    'Maximum cargo weight',
    'Maximum payload',
    'Maximum payload (fully electric)',
    'Maximum payload (hybrid-electric)',
    'Maximum payload at 0-5000 ft (0-1524 m)',
    'Maximum payload at 5000-13000 ft (1524-3962 m)',
    'Maximum payload weight',
    'Maximum payload weight (for the pilot)',
    'Maximum payload piston powertrain',
    'Payload',
    'Payload capabilities',
    'Payload weight',
    'Payload weight range',
    'Typical mission payload weight',
    'Useful load',
    'Vertical takeoff useful load',
    'Estimated maximum payload',
    'Estimated payload'
]

cargo = [
    'Cargo',
    'Cargo capacity',
    'Cargo maximum payload',
    'Cargo weight',
    'Cargo weight load'
]

max_speed = [
    'Max speed',
    'Maximum flight speed',
    'Maximum speed',
    'Maximum speed (approximately)',
    'Maximum speed (level flight)',
    'Maximum speed (piston engine)',
    'Racing speed',
    'Top speed',
    'Estimated maximum speed',
    'Flight maximum speed'
]

speed = [
    'Speed',
    'Flying speed'
]

min_speed = [
    'Minimum Speed',
    'Minimum speed'
]

max_cruise_speed = [
    'Maximum cruise speed',
    'Maximum cruise speed (air)',
    'eVTOL aircraft cruise speed',
    'Flying cruise speed',
    'Flight cruise speed',
    'Flight cruising speed',
    'Expected cruise speed',
    'Fast cruise speed',
    'Cruise speed (hybrid-electric)',
    'Cruise speed (target)',
    'Cruise speed for maximum range',
    'Cruise speed with best range (piston engine)',
    'Cruise speed with turbine engine',
    'Cruising speed (eVTOL)',
    'Cruise speed (air)',
    'Cruise speed (battery packs)',
    'Cruise Speed',
    'Cruise Speed (estimated)',
    'Cruise speed (estimated)',
    'Cruise speed',
    'Cruising speed',
    'Estimated Cruise Speed',
    'Driving cruise speed',
    'Estimate Cruise speed'
]

max_airspeed = [
    'Maximum airspeed',
]

max_altitude = [
    'Maximu altitude',
    'Maximum Altitude',
    'Maximum altitude (hybrid-electric)',
    'Maximum altitude',
    'Maximum altitude (fully electric)',
    'Top altitude',
    'Altitude',
    'Maximum VTOL takeoff altitude'
]

max_cruise_altitude = [
    'Maximum cruise altitude',
    'Normal cruise altitude',
    'Cruise altitude',
    'Cruising altitude',
    'Estimated Cruise altitude',
    'Cruise Altitude'
]

range = [
    'Normal range',
    'Maximum Range',
    'Maximum estimated range',
    'Maximum range',
    'Maximum range (+ 30 min reserve)',
    'Maximum range (hybrid-electric',
    'Maximum range (air)',
    'Maximum range (airborne)',
    'Maximum range (fully electric)',
    'Hybrid-Electric Range',
    'Hybrid-electric range',
    'Range',
    'Range (air)',
    'Range (battery packs)',
    'Range (estimated)',
    'Range (hybrid-electric)',
    'Range for flying',
    'Range hydrogen fuel cells',
    'Typical mission range (+ 30 min reserve)',
    'VTOL maximum range',
    'VTOL range',
    'eVTOL aircraft range',
    'Flying range',
    'Flying length',
    'Flight maximum range',
    'Flight range',
    'Expected range',
    'Estimated Range',
    'Estimated range',
    'All-electric Range',
    'Electric range',
    'Range with batteries',
    'Range with payload',
    'Range with turbine engine'
]

range_without_payload = [
    'Ferry range (Extended range with no passengers or cargo)',
    'Range without payload'
]

rate_of_climb = [
    'Rate of climb',
    'Estimated rate of climb'
]

reverse_time = [
    'Reserve time',
]

number_of_rotors = [
    'Number of rotors',
]

propeller_count = [
    'Propeller Count',
]

operating_temperature = [
    'Operating temperature',
    'Operating temperatures'
]

pilot = [
    'Pilot',
    'Pilot type',
    'Piloting'
]

price = [
    'Pricing',
    'Estimated retail price'
]
